// Schellings Segregation Model
// Code by Benedikt Stefansson, <benedikt@ucla.edu>. 
// First version July 1997
// Second version February 1998

#import "ModelSwarm.h"
#import <simtools.h>

@implementation ModelSwarm

// createBegin: here we set up the default simulation parameters.
+createBegin: (id) aZone {
  ModelSwarm * obj;
  id <ProbeMap> probeMap;

  // First, call our superclass createBegin. This makes sure that
  // the obj(ect) - the ModelSwarm - is created correctly
  obj = [super createBegin: aZone];

  // Now fill in various simulation parameters with default values.
  obj->worldSize = 50;
  obj->neighborhood_type = strdup("vonneuman");
  obj->fractionVacant = 0.2;
  obj->fractionBlue = 0.5;
  obj->blueToleranceUpper=0.50;
  obj->blueToleranceLower=0.25;
  obj->redToleranceUpper=0.50;
  obj->redToleranceLower=0.25;
  obj->blueToleranceLower=0.25;
  obj->randomSeed=882246;
  
  // Then create an EmptyProbeMap to hold the individual
  // probes for each simulation parameter
  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // Add in a bunch of probes one per simulation parameter
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldSize"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "neighborhood_type"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "fractionVacant"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "fractionBlue"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "blueToleranceUpper"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "blueToleranceLower"
				    inClass: [self class]]];
 [probeMap addProbe: [probeLibrary getProbeForVariable: "redToleranceUpper"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "redToleranceLower"
				    inClass: [self class]]];
 [probeMap addProbe: [probeLibrary getProbeForVariable: "randomSeed"
				    inClass: [self class]]];
  [probeMap addProbe: [[probeLibrary getProbeForMessage: "saveParameters"
			                        inClass: [self class]]
			                  setHideResult: 1]];
  [probeMap addProbe: [[probeLibrary getProbeForMessage: "loadParameters"
			                        inClass: [self class]]
			                  setHideResult: 1]];

  // Now install our custom probeMap into the probeLibrary.
  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;
}

-createEnd {
  return [super createEnd];
}

// Methods associated with probe above, to 
// save or load the parameters from a file
-saveParameters {
  
  [ObjectSaver save: self toFileNamed: "model.setup"];
  return self;
}
  
-loadParameters {
  [ObjectLoader load: self fromFileNamed: "model.setup"];
  return self;
}


-buildObjects {
  int x,y;
  int color;  
  double tolerance;

  [super buildObjects];

  // We need a list for the agents
  agentList=[List create:[self getZone]];

  // Now create a 2d array to hold their location info
  world=[SchellingWorld createBegin: [self getZone]];
  [world setSizeX: worldSize Y: worldSize];
  [world setModelSwarm: self];
  world=[world createEnd];

  // Also some random number generators
  aGenerator=[PMMLCG1 create: [self getZone] setStateFromSeed: randomSeed];
  uniformDouble=[UniformDouble create: [self getZone] setGenerator: aGenerator];
  uniformInteger=[UniformInteger create: [self getZone] setGenerator: aGenerator];
  
 
  // Then proceed to create the people in the world
 for(x=0;x<worldSize;x++) {
   for(y=0;y<worldSize;y++) {
     id person;
     id neighborhood;
     // With probability 1-fractionVacant put an
     // agent on the square and with probability
     // fractionBlue make an agent of that color
     if([self getRandomDouble]>=fractionVacant) {
       if([self getRandomDouble]<=fractionBlue) {
	 // Here we initialize variables for a blue agent
	 color=10;
	 tolerance=[self getRandomDoubleMin: blueToleranceLower
			 Max: blueToleranceUpper];
       }
       else {
	 // Here  we initialize variables for a red agent
	 color=11;
	 tolerance=[self getRandomDoubleMin: redToleranceLower
			 Max: redToleranceUpper];
       }
       
       // Each agent will have a neighborhood object
       // which encapsulates the "neighborhood" rule       
       neighborhood=[Neighborhood create:[self getZone]];
       [neighborhood setX: x Y: y];
       [neighborhood setWorld: world];
       if(neighborhood_type[0]=='v') 
	 [neighborhood setType: 1];
       else 
	 [neighborhood setType: 2];
		 
       // Finally we create the actual person
       person=[Person create: [self getZone]];
       [person setWorld: world];
       [person setNeighborhood: neighborhood];
       [person setAgentColor: color];
       [person setTolerance: (float)tolerance];

       // Now that the person has been created and
       // initialized we have to add it to the world
       // and the agentList which is used by the schedule
       [world putObject: person atX: x Y: y];
       [agentList addLast: person];	
     }
   }
 }

 return self;
}


-getAgentList {
  return agentList;
}

-getWorld {
  return world;
}

-(int)getWorldSize {
  return worldSize;
}


-(double)getRandomDoubleMin: (double) min Max: (double) max {

  double num;

  if(min==max)
    num=max;
  else
    num=[uniformDouble getDoubleWithMin: min withMax: max];
  
  return num;
}

-(double)getRandomDouble {
  double num;

  num=[uniformDouble getDoubleWithMin: 0.0 withMax: 1.0];
  
  return num;
}

-(int)getRandomIntMin: (int) min Max: (int) max {
  int num;

  if(min==max)
    num=max;
  else
    num=[uniformInteger getIntegerWithMin: min withMax: max];

  return num;
}

		
-buildActions {
  [super buildActions];
      
  modelSchedule = [Schedule createBegin: [self getZone]];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createActionForEach: agentList message: M(step)];

  return self;
}

-activateIn: (id) swarmContext {
  // First, activate ourselves via the superclass activateIn: method.
  // Just pass along the context: the activity library does the right thing.
  [super activateIn: swarmContext];

  // Now activate our own schedule.
  [modelSchedule activateIn: self];

  // Finally, return our activity.
  return [self getSwarmActivity];
}

@end











